open Lexer_types

type context =
    { b_flag : bool; em_flag : bool; i_flag : bool; s_flag : bool; tt_flag : bool;
      u_level : int; size : int option; color_val : color option }

let changed_context ctx tg =
  match tg with
    B_tag -> { ctx with b_flag = true }
  | EM_tag -> if ctx.s_flag then ctx else { ctx with em_flag = not ctx.em_flag }
  | I_tag -> { ctx with i_flag = true }
  | PL_tag -> { ctx with u_level = 0; s_flag = false; em_flag = false; i_flag = false;
		b_flag = false; tt_flag = false }
  | S_tag -> { ctx with s_flag = true; em_flag = false }
  | TT_tag -> { ctx with tt_flag = true }
  | U_tag -> if ctx.u_level < 3 then { ctx with u_level = ctx.u_level + 1 } else ctx
  | Size_tag s -> { ctx with size = Some s }
  | Color_tag c -> { ctx with color_val = Some c }

let root_context = 
	{ b_flag = false;
	  em_flag = false;
	  i_flag = false;
	  tt_flag = false;
	  s_flag = false;
	  size = None;
	  color_val = None;
	  u_level = 0 }

let contexts_render_ws_same c d =
  c.tt_flag = d.tt_flag && c.size = d.size && c.u_level = d.u_level && (c.u_level = 0 || c.color_val = d.color_val)

let color_name c =
  match c with
    Red -> "r" | Green -> "g" | Blue -> "b" | Cyan -> "c" | Magenta -> "m" | Yellow -> "y" | Black -> "k" | White -> "w"

let print_context c =
  print_string
    ((if c.b_flag then "B " else "") ^ (if c.em_flag then "EM " else "") ^ (if c.i_flag then "I " else "") ^
     (if c.s_flag then "S " else "") ^ (if c.tt_flag then "TT " else "") ^ "U" ^ (string_of_int c.u_level) ^ " " ^
     (match c.size with
       None -> "SR "
     | Some s -> "S" ^ string_of_int s) ^
     (match c.color_val with
       None -> "CR"
     | Some clr -> "C" ^ color_name clr))
