open List
open Lexer_types
open Lexer

open Context
open Util

let matchingCloseOpenTokens = function
  | (Close_token t, Open_token t') -> t = t'
  | _ -> false

let emitPendingStringToken emitted_tokens = function
  | Some xs -> (Text_token xs)::emitted_tokens
  | None    -> emitted_tokens

let token_is_tagtoken = function
  | Open_token _ | Close_token _ -> true
  | _ -> false

let isMatchingCloseTag tag = function
  | (true,t)::_ -> t=tag
  | _ -> false

let addToPendingStringToken x txt = match x with
  | Some txt' -> Some (txt' ^ txt)
  | None -> Some txt

(* exported *)
let lexbuf_to_tokenlist lexbuf = 
 let rec retokenize' 
		emitted_tokens 
		current_context
		pending_tags_stack
		pending_string_token
		this_token =
  let next_token = Lexer.scan lexbuf in
  match this_token with
    | EOF ->
	emitPendingStringToken emitted_tokens pending_string_token
	
    | Close_token tag when matchingCloseOpenTokens (this_token,next_token) ->
	retokenize' 
		emitted_tokens 
		current_context
		pending_tags_stack
		pending_string_token
		(Lexer.scan lexbuf)			(* fetch new token *)


      | Close_token tag when isMatchingCloseTag tag pending_tags_stack ->
	  retokenize' 
		emitted_tokens 
		current_context
		(tl pending_tags_stack) 
		pending_string_token
		next_token

	(* verwende isMatchingCloseTag *)

(*
###
      | Close_token tag when (match pending_tags_stack with 
				| (true,t)::_ -> t=tag
			 	| _ -> false) ->
###
*)

      | Close_token tag ->
	  retokenize' 
		(this_token::(emitPendingStringToken emitted_tokens pending_string_token))
		(tl current_context) 
		(tl pending_tags_stack)
		None
		next_token

      | Open_token tag ->
	  let old_state = hd current_context in
	  let new_state = changed_context old_state tag in
	    if new_state = old_state then
	      retokenize' 
			emitted_tokens 
			current_context
			((true,tag)::pending_tags_stack)
			pending_string_token
			next_token
	    else
	      retokenize' 
			(this_token::(emitPendingStringToken emitted_tokens pending_string_token))
			(new_state::current_context) 
			((false,tag)::pending_tags_stack)
			None
			next_token

      | Text_token txt ->
	  retokenize' 
		emitted_tokens 
		current_context
		pending_tags_stack
		(addToPendingStringToken pending_string_token txt)
		next_token

      | WS_token ws ->
	  let current_state = hd current_context in
	  if current_state.tt_flag then
	    (* combine whitespaces *)
	    retokenize' 
		emitted_tokens 
		current_context
		pending_tags_stack
		(addToPendingStringToken pending_string_token ws)
		next_token
	  else if pending_string_token = None then 	(* ws after a tag *)
	    retokenize' 
		((WS_token " ")::emitted_tokens) 
		current_context
		pending_tags_stack
		None
		next_token
	  else if token_is_tagtoken next_token then
	    retokenize' 
		((WS_token " ")::(emitPendingStringToken emitted_tokens pending_string_token))
		current_context
		pending_tags_stack
		None
		next_token
	  else
	    retokenize' 
		emitted_tokens 
		current_context
		pending_tags_stack
		(addToPendingStringToken pending_string_token " ")
		next_token

  in retokenize' [] [root_context]  [] None (Lexer.scan lexbuf)

