type color =
    Red
  | Green
  | Blue
  | Cyan
  | Magenta
  | Yellow
  | Black
  | White

type tag =
    B_tag
  | EM_tag
  | I_tag
  | PL_tag
  | S_tag
  | TT_tag
  | U_tag
  | Size_tag of int
  | Color_tag of color

type token =
    Text_token of string
  | WS_token of string
  | Open_token of tag
  | Close_token of tag
  | EOF


let colorToString = function
  | Red -> "r"
  | Green -> "g"
  | Blue -> "b"
  | Cyan -> "c"
  | Magenta -> "m"
  | Yellow -> "y"
  | Black -> "k"
  | White -> "w"

let tagToString = function
  | B_tag -> "B"
  | EM_tag -> "EM"
  | I_tag -> "I"
  | PL_tag -> "PL"
  | S_tag -> "S"
  | TT_tag -> "TT"
  | U_tag -> "U"
  | Size_tag s -> string_of_int s
  | Color_tag c -> colorToString c

let opentagToString = function
  | B_tag  -> "<B>"
  | EM_tag -> "<EM>"
  | I_tag  -> "<I>"
  | PL_tag -> "<PL>"
  | S_tag  -> "<S>"
  | TT_tag -> "<TT>"
  | U_tag  -> "<U>"
  | Size_tag s -> "<" ^ string_of_int s ^ ">"
  | Color_tag c -> "<" ^ colorToString c ^ ">"

let closetagToString = function
  | B_tag  -> "</B>"
  | EM_tag -> "</EM>"
  | I_tag  -> "</I>"
  | PL_tag -> "</PL>"
  | S_tag  -> "</S>"
  | TT_tag -> "</TT>"
  | U_tag  -> "</U>"
  | Size_tag s -> "</" ^ string_of_int s ^ ">"
  | Color_tag c -> "</" ^ colorToString c ^ ">"


let print_tok = function
  | Text_token t -> print_string "text "; print_string t; print_newline()
  | WS_token t -> print_string "whitespace \""; print_string t; print_string "\""; print_newline()
  | Open_token tg -> print_string "open "; print_string (tagToString tg); print_newline()
  | Close_token tg -> print_string "close "; print_string (tagToString tg); print_newline()
  | EOF -> failwith "print_tok: EOF"

let emitToken = function
  | Text_token s -> print_string s 
  | WS_token s -> print_string s
  | Open_token tag -> Printf.printf "<%s>" (tagToString tag)
  | Close_token tag -> Printf.printf "</%s>" (tagToString tag)
  | EOF -> failwith "emitToken: EOF"


let tagIsSize = function
  | Size_tag _ -> true
  | _ -> false

