(* In this module inner whitespaces and superfluous state transitions are eliminated. *)

open List
open Util
open Debug
open Lexer_types
open Lexer
open Context
open Lexer_simp
open Parser
open ContMonad

let firstlastpartition = function
  | [] -> failwith "firstlastpartition (1)"
  | [_] -> failwith "firstlastpartition (2)"
  | first::xs ->
      let rec firstlastpartition' middle = function
	| []  -> failwith "firstlastpartition' (1)"
	| [last] -> (first, rev middle, last)
	| x::xs -> firstlastpartition' (x::middle) xs
      in firstlastpartition' [] xs

let whitespaceToCombElem = function
  | WhiteSpace _ as ws -> ([ws], [])
  | x		       -> ([], [x])

let rec joinadjacentwhitespaces = function
  | [] -> []
  | (WhiteSpace _ as ws1)::(WhiteSpace _ as ws2)::xs -> joinadjacentwhitespaces (ws1::xs)
  | x::xs -> x::(joinadjacentwhitespaces xs)

let rec littlesimplifierM prev_state tree = match tree with
  | Text x -> unitM (Text x)
  | WhiteSpace x -> unitM (WhiteSpace x)
  | Node(_,tags,successors) ->
      let current_state = List.fold_left changed_context prev_state tags in
	mapM (littlesimplifierM current_state) successors >>= fun xs0 ->
	let xs1 = concat (map treenode_to_nodelist xs0) in
	let xs' = if not current_state.tt_flag && current_state.u_level = 0 then
		    joinadjacentwhitespaces xs1
		  else
		    xs1
	in
	  if prev_state = current_state then
	    unitM (Node(true,[],xs'))
	  else
	    if  not prev_state.tt_flag && 
		prev_state.u_level = current_state.u_level && 
		not (exists tagIsSize tags) && 
		not current_state.tt_flag && 
		current_state.u_level = 0 then

		(match xs' with
		  | [] -> 
			unitM (Node(true,[],[]))
		  | [WhiteSpace _ as ws1] -> 
			unitM (ws1)  (* eliminate tag-node *)
		  | [x] -> 
			unitM tree   (* assumes that tree is an optimizable node *)
		  | xs -> 
		      let (first,middle,last) = firstlastpartition xs in
			let (up1,first') = whitespaceToCombElem first 
			and (up2,last')  = whitespaceToCombElem last in
			let middle' = first' @ middle @ last' in
			let middle'' = joinadjacentwhitespaces middle' in
		 	  unitM (Node(true,[], up1 @ [Node(true,tags,middle'')] @ up2)))
	    else 
	      unitM (Node(true,tags,xs'))


(* exported *)
let littlesimplifier initial_context tree =
  ContMonad.runM (littlesimplifierM initial_context) tree

