(*
 * Copyright (c) 2001 Stefan Kral
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)

open List
open NonDetMonad

let mapM f xs =
  let rec mapM' bag = function
    | []    -> unitM (rev bag) 
    | x::xs -> f x >>= fun x' ->
		 mapM' (x'::bag) xs
  in mapM' [] xs

let consM x xs = unitM (x::xs)

let memberM = function
  | []    -> failM
  | x::xs -> 
      let rec memberM' el = function  (* eliminates superfluous choicepoint *)
	| []    -> unitM el
	| x::xs -> (fun _ -> unitM el) ||| (fun _ -> memberM' x xs)
      in memberM' x xs

let selectM = function
  | []    -> failM
  | x::xs -> 
      let rec selectM' el = function  (* eliminates superfluous choicepoint *)
        | [] -> 
	    unitM (el,[])
        | x::xs as xxs -> 
	    (fun _ -> unitM (el,xxs)) |||
	    (fun _ -> selectM' x xs >>= fun (z,zs) -> unitM (z,el::zs))
      in selectM' x xs


let select3M xs = 
  let rec select3M' pre_rev = function
    | [] -> failM
    | x::xs ->
	(   (fun _ -> unitM (pre_rev,x,xs))
	||| (fun _ -> select3M' (x::pre_rev) xs)
	)
  in select3M' [] xs

let select4M xs = 
  let rec select4M' pre_rev = function
    | [] -> failM
    | [_] -> failM
    | x1::x2::xs ->
	(   (fun _ -> unitM (pre_rev,x1,x2,xs))
	||| (fun _ -> select4M' (x1::pre_rev) (x2::xs))
	)
  in select4M' [] xs

let select5M xs =
  let rec select5M' pre_rev = function
    | [] -> failM
    | [_] -> failM
    | [_;_] -> failM
    | x1::x2::x3::xs ->
	(   (fun _ -> unitM (pre_rev,x1,x2,x3,xs))
	||| (fun _ -> select5M' (x1::pre_rev) (x2::x3::xs))
	)
  in select5M' [] xs


let rec iterM f = function
  | []    -> unitM ()
  | x::xs -> f x >>= fun _ -> iterM f xs


