open List
open Util
open Debug
open Lexer_types
open Context
open ContMonad

type treenode =
  | Text of string
  | WhiteSpace of string
  | Node of bool * tag list * treenode list
	(* optimizable, list of tags, list of successor nodes *)


let rec emitTreeM = function
  | Text s -> 
      unitM (print_string s)
  | WhiteSpace(s) -> 
      unitM (print_string s)
  | Node(_,tags,successors) ->
      iter (fun tag -> print_string (opentagToString tag)) tags;
      iterM emitTreeM successors >>= fun _ ->
        unitM (iterrev (fun tag -> print_string (closetagToString tag)) tags)

let emitTree tree =
  ContMonad.runM emitTreeM tree

let treenode_to_nodelist = function
  | Text _ as node -> [node]
  | WhiteSpace _ as node -> [node]
  | Node(_,[],xs) -> xs
  | Node _ as node -> [node]

let rec buildtree'' tag bag = function
  | [] -> failwith "buildtree'' : not wellformed (unexpected EOF)!"
  | t::ts ->
      match t with
        | Text_token s -> buildtree'' tag ((Text s)::bag) ts
        | WS_token s -> buildtree'' tag ((WhiteSpace s)::bag) ts
        | Close_token tag' ->
	    buildtree'' tag' [] ts >>= fun (successors,ts') ->
	      buildtree'' tag ((Node(true,[tag'],successors))::bag) ts'
        | Open_token t when t = tag -> 
	    unitM (bag,ts)
        | Open_token _ -> failwith "buildtree'': not wellformed!"
        | EOF -> failwith "buildtree'' : not wellformed (unexpected EOF)!"

and buildtree' bag = function
  | [] -> unitM bag
  | t::ts ->
      match t with
	| Text_token s -> buildtree' ((Text s)::bag) ts
	| WS_token s -> buildtree' ((WhiteSpace s)::bag) ts
	| Close_token tag ->
	    buildtree'' tag [] ts >>= fun (successors,ts') ->
	      buildtree' ((Node(true,[tag],successors))::bag) ts'
	| Open_token _ -> failwith "buildtree': not wellformed!"
	| EOF -> failwith "buildtree': EOF"

let buildtree tokens =
  Node(true,[],ContMonad.runM (buildtree' []) tokens)
